@@ -7,59 +7,18 @@ import uuid
 from datetime import datetime as dt
 
 import aiohttp
-from pubnub.callbacks import SubscribeCallback
-from pubnub.enums import PNReconnectionPolicy, PNStatusCategory
+from pubnub.enums import PNReconnectionPolicy
 from pubnub.pnconfiguration import PNConfiguration
 from pubnub.pubnub_asyncio import PubNubAsyncio
 
 from .containers import (
     AuthorizationInfo,
-    SnooData,
     SnooDevice,
     SnooStates,
 )
 from .exceptions import InvalidSnooAuth, SnooAuthException, SnooDeviceError
 from .pubnub_async import SnooPubNub
 
-
-class MySubscribeCallback(SubscribeCallback):
-    def __init__(self, snoo_instance, device_id, message_type):
-        self.snoo_instance = snoo_instance
-        self.device_id = device_id
-        self.message_type = message_type
-
-    def status(self, pubnub, status):
-        if status.category == PNStatusCategory.PNUnexpectedDisconnectCategory:
-            pass  # Handle unexpected disconnect
-        elif status.category == PNStatusCategory.PNConnectedCategory:
-            pass  # This is expected for a subscribe, this means we are connected and subscribing
-        elif status.category == PNStatusCategory.PNReconnectedCategory:
-            pass  # Happens as part of a reconnect process
-        elif status.category == PNStatusCategory.PNDisconnectedCategory:
-            pass
-        elif status.is_error():
-            _LOGGER.error(f"Subscription error: {status}")
-
-    def presence(self, pubnub, presence):
-        pass  # Handle presence events if needed
-
-    def message(self, pubnub, message):
-        if self.message_type == "Activity":
-            if "system_state" in message.message:
-                try:
-                    data = SnooData.from_dict(message.message)
-                    _LOGGER.debug(data)
-                    self.snoo_instance.data_map[self.device_id] = data
-                    if self.device_id in self.snoo_instance.subscription_functions:
-                        self.snoo_instance.subscription_functions[self.device_id]()
-                except Exception as e:
-                    _LOGGER.error(f"Error processing message: {e}, Message: {message.message}")
-            else:
-                _LOGGER.debug(f"[{self.device_id}] Got message on {self.message_type} of {message.message}")
-        else:
-            _LOGGER.debug(f"[{self.device_id}] Got message on {self.message_type} of {message.message}")
-
-
 _LOGGER = logging.getLogger(__name__)
 
 
@@ -127,6 +86,7 @@ class Snoo:
         self.subscription_functions = {}
         self.data_map = {}
         self.pubnub_instances: dict[str, SnooPubNub] = {}
+        self.reauth_task: asyncio.Task | None = None
 
     async def refresh_tokens(self):
         # TODO: Figure out hwo to get this to work and not do a serializaiton exception
@@ -194,35 +154,39 @@ class Snoo:
                 except asyncio.CancelledError:
                     pass
         self.pubnub_instances = {}
+        self.reauth_task.cancel()
 
     def publish_callback(self, result, status):
         if status.is_error():
             _LOGGER.warning(f"Message failed with {status.status_code}, {status.error_data.__dict__}")
 
-    def send_command(self, command: str, device: SnooDevice, **kwargs):
+    async def send_command(self, command: str, device: SnooDevice, **kwargs):
         ts = int(dt.now().timestamp() * 10_000_000)
-        self.pubnub.publish().channel(f"ControlCommand.{device.serialNumber}").message(
-            {"ts": ts, "command": command, **kwargs}
+        await (
+            self.pubnub.publish()
+            .channel(f"ControlCommand.{device.serialNumber}")
+            .message({"ts": ts, "command": command, **kwargs})
+            .future()
         )
 
-    def start_snoo(self, device: SnooDevice):
-        self.send_command("start_snoo", device)
+    async def start_snoo(self, device: SnooDevice):
+        await self.send_command("start_snoo", device)
 
-    def stop_snoo(self, device: SnooDevice):
-        self.send_command("go_to_state", device, **{"state": "ONLINE", "hold": "off"})
+    async def stop_snoo(self, device: SnooDevice):
+        await self.send_command("go_to_state", device, **{"state": "ONLINE", "hold": "off"})
 
-    def set_level(self, device: SnooDevice, level: SnooStates):
-        self.send_command("go_to_state", device, **{"state": level.value, "hold": "off"})
+    async def set_level(self, device: SnooDevice, level: SnooStates):
+        await self.send_command("go_to_state", device, **{"state": level.value, "hold": "off"})
 
-    def set_sticky_white_noise(self, device: SnooDevice, on: bool):
-        self.send_command(
+    async def set_sticky_white_noise(self, device: SnooDevice, on: bool):
+        await self.send_command(
             "set_sticky_white_noise",
             device,
             **{"state": "on" if on else "off", "timeout_min": 15},
         )
 
-    def get_status(self, device: SnooDevice):
-        self.send_command("send_status", device)
+    async def get_status(self, device: SnooDevice):
+        await self.send_command("send_status", device)
 
     async def auth_amazon(self):
         r = await self.session.post(self.aws_auth_url, data=self.aws_auth_data, headers=self.aws_auth_hdr)
@@ -247,7 +211,7 @@ class Snoo:
             snoo_expiry = snoo_token["expiresIn"] / 1.5
             snoo_token = snoo_token["snoo"]["token"]
             self.tokens = AuthorizationInfo(snoo=snoo_token, aws_access=access, aws_id=_id, aws_refresh=ref)
-            asyncio.create_task(self.schedule_reauthorization(snoo_expiry))
+            self.reauth_task = asyncio.create_task(self.schedule_reauthorization(snoo_expiry))
         except InvalidSnooAuth as ex:
             raise ex
         except Exception as ex:
